
/***************************************************
* By CCSC compiler
*  File Name: 20200630_K630_MK508B_1823.c
*  Rev 1.0
 * rev 1.1
*  Description: talking distance meter
 * RC0 in na
*  RC1 in na
*  RC2 out P1D
*  RC3 out P1C
*  RC4 out P1B
*  RC5 out P1A
*  RA0 in na
 * RA1 in na
 * RA2 in AD freq
 * RA3 in na
*  RA4 out LED
 * RA5 in SW
*  COPYRIGHT 2020 MYCOMKITS.COM, owned by CNET LIMITED
$B$3$N%W%m%0%i%`$NCx:n8"$O!"@=:n<T!V%^%$%3%s%-%C%H%I%C%H%3%`1?1D!!M-8B2q<R%/%M%C%H!W$K5"B0$7$^$9!#(B
$B2f!9$OCx:n8"$rJ|4~$7$F$$$^$;$s$,!"Ev%W%m%0%i%`$r;H$C$?3X=,$NCf$G%W%m%0%i%`$r<+M3$KJQ99$7$F$*;H$$$/$@$5$$!#(B
The copyright of this program belongs to the producer "MYCOMKITS.COM$B!I(Bmanagement company CNET LIMITED". 
We DO not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16f1823.h>
#DEVICE ADC=10
#fuses   INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOLVP, NOPROTECT, NODEBUG
#fuses   NOCPD, NOCLKOUT, NOWRT, NOIESO, NOFCMEN, PLL_SW, NOSTVREN
#use   delay(CLOCK = 8000000)
#use pwm(CCP1, FREQUENCY=3.9k)
#ROM getenv("EEPROM_ADDRESS") = {0, 0, 0, 0, 0, 0, 0, 0}// get 8 bytes

//
#use   fast_io(a)
#use   fast_io(c)

//
#define Amode 0xEF         //port A all input
#define Cmode 0xC3         //port C all input except 1, 2, 5

//
#byte OSCCON = 0x099
#byte WPUC = 0x20E
#byte OPTION_REG = 0x095
#byte PCON = 0x096
#bit T1IF = 0x011.0
#bit T1ON = 0x018.0
#define P1A PIN_C5
#define P1B PIN_C4
#define P1C PIN_C3
#define P1D PIN_C2
#define LED PIN_A4
#define SW PIN_A5

// variables

long value = 512;   // max 1023
long value_old = 512;   // max 1023
long value_s[9];
int freq=0;
int set_freq=1;
int flag9bit=0;   //0 FOR 8bit, 1 for 9bit
int pwm_setflag_p=0;
int pwm_setflag_n=0;

//
#SEPARATE
void initializing();

#SEPARATE
void get_pot();

#SEPARATE
void get_freq();

#SEPARATE
void set_frequency();

#SEPARATE
void set_pwm();

#SEPARATE
void led_brink(INT x);

#SEPARATE
void test();

// main function
void main()
{
   initializing () ;
   
   // main loop
   WHILE (1)
   {
      get_freq (); // check SWITCH and change freq
      
      get_pot (); // get pmw value
      set_pwm () ;
   }
}

//
#SEPARATE
void initializing() 
{
   set_tris_a (Amode); //
   set_tris_c (Cmode); //
   //
   port_a_pullups (0x20); //PIN_A5
   setup_ccp1 (CCP_PWM_FULL_BRIDGE|CCP_PWM_H_H) ;
   output_low (LED) ;
   output_low (P1A) ;
   output_low (P1B) ;
   output_low (P1C) ;
   output_low (P1D) ;
   set_pwm1_duty (0) ;

   //
   setup_comparator (NC_NC_NC_NC) ;
   setup_dac (DAC_OFF) ;

   // A / D converter initialize
   setup_adc_ports (sAN2|VSS_VDD); // RA4, gnd to 5.0V
   setup_adc (ADC_CLOCK_DIV_8); // 2usec
   
   // Timer1 Setup
   setup_timer_1 (T1_INTERNAL|T1_DIV_BY_8); // Fosc / 4, 1:8, Enables Timer1, resolution 4us (0.25MHz)
   set_timer1 (0) ;
   T1IF = 0;
   T1ON = 1;
   
   set_freq = read_eeprom (0);
   IF ( (set_freq > 6)|| (set_freq == 0)) set_freq = 4; // 1900hz
   value = 512;
   value_old = 0; // max 1023
   set_frequency () ;
   
   set_adc_channel (2); //must wait 65u, AN3, RA4
   delay_us (100) ;
   value = read_adc (); // 0 to 1023, 0V to 5V
   INT i;
   FOR (i = 0; i<8; i++)
   {
      value_s[i] = value;
   }

   get_pot () ;
}

// get repeat time value from potentiometer
#SEPARATE
void get_pot()
{
   // moving average
   INT i;
   FOR (i = 7; i>0; i--)
   {
      value_s[i] = value_s[i - 1];
   }

   set_adc_channel (2); //must wait 65u, AN3, RA4
   delay_us (100) ;
   value_s[0] = read_adc (); // 0 to 1023, 0V to 5V
   FOR (i = 0; i<8; i++)
   {
      value = value + value_s[i];
   }

   value = value / 8;
}

//
#SEPARATE
void get_freq()
{
   // 1:244Hz
   // 2:488Hz
   // 3:976Hz
   // 4:1950Hz
   // 5:3900Hz
   // 6:7800Hz
   IF (input (SW) == 0)
   {
      delay_ms (20) ;

      IF (input (SW) == 0)
      {
         set_freq = set_freq + 1;
         IF (set_freq > 6) set_freq = 1;
         write_eeprom (0, set_freq);
         set_frequency () ;
      }
   }
}

//
#SEPARATE
void set_frequency()
{
   SWITCH (set_freq)
   {
      CASE 1:
      pwm_set_frequency (244) ;
      flag9bit = 1;
      
      BREAK;

      CASE 2:
      pwm_set_frequency (488) ;
      flag9bit = 0;
      
      BREAK;

      CASE 3:
      pwm_set_frequency (976) ;
      flag9bit = 1;
      
      BREAK;

      CASE 4:
      pwm_set_frequency (1900) ;
      flag9bit = 0;
      
      BREAK;

      CASE 5:
      pwm_set_frequency (3906) ;
      flag9bit = 1;
      
      BREAK;

      CASE 6:
      pwm_set_frequency (7812) ;
      flag9bit = 0;
      
      BREAK;

      DEFAULT:
      BREAK;
   }

   value_old = 2000;
   set_pwm () ;
   led_brink (set_freq) ;
   write_eeprom (0, set_freq);
}

//
#SEPARATE
void led_brink(INT n)
{
   INT i;
   FOR (i = 0; i<n; i++)
   {
      output_high (LED) ;
      delay_ms (200) ;
      output_low (LED) ;
      delay_ms (200) ;
   }

}

//
#SEPARATE
void set_pwm()
{
   LONG value_temp = 0;

   IF (value != value_old)
   {
      
      IF (value >= 547)
      {
         pwm_setflag_n = 0;

         IF (pwm_setflag_p == 0)
         {
            setup_ccp1 (CCP_PWM_FULL_BRIDGE|CCP_PWM_H_H) ;
            pwm_setflag_p = 1;
         }

         value_temp = value - 547; //0 to 476
         value_temp = value_temp * 22; //0 to 10472
         value_temp = value_temp / 20; //0 to 524
         IF (value_temp > 511) value_temp = 511;
         IF (flag9bit == 0) value_temp = value_temp / 2;
         set_pwm1_duty (value_temp) ;
      }

      else IF (value <= 477)
      {
         pwm_setflag_p = 0;

         IF (pwm_setflag_n == 0)
         {
            setup_ccp1 (CCP_PWM_FULL_BRIDGE_REV|CCP_PWM_H_H) ;
            pwm_setflag_n == 1;
         }

         value_temp = 477 - value; //0 to 477
         value_temp = value_temp * 22; //0 to 10494
         value_temp = value_temp / 20; //0 to 524
         IF (value_temp > 511) value_temp = 511;
         IF (flag9bit == 0) value_temp = value_temp / 2;
         set_pwm1_duty (value_temp) ;
      }

      ELSE
      {
         value_temp = 0;
         set_pwm1_duty (0) ;
      }

      value_old = value;
   }
}

//
#SEPARATE
void test()
{
   setup_ccp1 (CCP_PWM_FULL_BRIDGE|CCP_PWM_H_H) ;
   
   INT i;

   WHILE (1)
   {
      FOR (i = 0; i < 7; i++)
      {
         set_pwm1_duty (i * 10) ;
         led_brink (set_freq) ;
         delay_ms (200) ;
      }
   }
}

